'''
Created on Nov 20, 2014

@author: Tara L Andrews

This module contains the exception classes used elsewhere in CollateX.
'''

class TokenError(Exception):
    pass

class UnsupportedError(Exception):
    pass