#!/usr/bin/env python
# -*- coding: utf-8 -*-

__author__ = 'Ronald Haentjens Dekker'
__email__ = 'ronald.dekker@huygens.knaw.nl'
__version__ = '2.0.0pre10'

from collatex.core_functions import Collation
from collatex.core_functions import collate
from collatex.core_functions import collate_pretokenized_json

__all__ = ["Collation", "collate", "collate_pretokenized_json"]


